#include <Servo.h>

#define X_STEP        2
#define Y_STEP        3
#define Z_STEP        4
#define X_DIR         5
#define Y_DIR         6
#define Z_DIR         7
#define STEPPER_EN    8

#define LIMIT_X       9
#define LIMIT_Y      10
#define LIMIT_Z      12

#define SPINDLE_PWM  11
#define SPINDLE_DIR  13

#define PROBE        A5
#define COOLANT      A3

#define EDM_PULSES   A4

#define STEPPER_COM_PAUSE      1
#define STEPPER_STEP_PAUSE   190

#define RETRACT              100

int retract_steps = 0;

Servo edm_pulses;  // create servo object to control a servo

int doHome = 0;

int edm_active = 0;

#define STEP_PAUSE_UP      1500
#define STEP_PAUSE_DOWN    1500

unsigned long lastRetract = 0;

void getHeightZ(){
  while(digitalRead(PROBE)){
    makeStepZ(0);
    delayMicroseconds(STEPPER_STEP_PAUSE);    
  }

  for(int i = 0; i < 2000; i++){
    makeStepZ(1);
    delayMicroseconds(STEPPER_STEP_PAUSE);      
  }  
}

void makeStepX(int dir){
  static int dirOld = -1;
  if(dir != dirOld){
    digitalWrite(X_DIR, dir);
    dirOld = dir;
  }
  digitalWrite(X_STEP, HIGH);
  delayMicroseconds(STEPPER_COM_PAUSE);
  digitalWrite(X_STEP, LOW);
}

void makeStepY(int dir){
  static int dirOld = -1;
  if(dir != dirOld){
    digitalWrite(Y_DIR, dir);
    dirOld = dir;
  }
  digitalWrite(Y_STEP, HIGH);
  delayMicroseconds(STEPPER_COM_PAUSE);
  digitalWrite(Y_STEP, LOW);
}

void makeStepZ(int dir){
  static int dirOld = -1;
  if(dir != dirOld){
    digitalWrite(Z_DIR, dir);
    dirOld = dir;
  }
  digitalWrite(Z_STEP, HIGH);
  delayMicroseconds(STEPPER_COM_PAUSE);
  digitalWrite(Z_STEP, LOW);
}

void homeAxes(){
  if( 0b00000001 & doHome){
    digitalWrite(STEPPER_EN, LOW);
    while(digitalRead(LIMIT_X)){
      makeStepX(1);
      delayMicroseconds(STEPPER_STEP_PAUSE);
    }
    delay(200);
    for(int i = 0; i < 2000; i++){
      makeStepX(0);
      delayMicroseconds(STEPPER_STEP_PAUSE);      
    }
    digitalWrite(STEPPER_EN, HIGH);
    doHome -= 1;
  }

  if( 0b00000010 & doHome){
    digitalWrite(STEPPER_EN, LOW);
    while(digitalRead(LIMIT_Y)){
      makeStepY(1);
      delayMicroseconds(STEPPER_STEP_PAUSE);
    }
    delay(200);
    for(int i = 0; i < 2000; i++){
      makeStepY(0);
      delayMicroseconds(STEPPER_STEP_PAUSE);      
    }
    digitalWrite(STEPPER_EN, HIGH);
    doHome -= 2;
  }

  if( 0b00000100 & doHome){
    digitalWrite(STEPPER_EN, LOW);
    while(digitalRead(LIMIT_Z)){
      makeStepZ(1);
      delayMicroseconds(STEPPER_STEP_PAUSE);
    }
    delay(200);
    for(int i = 0; i < 2000; i++){
      makeStepZ(0);
      delayMicroseconds(STEPPER_STEP_PAUSE);      
    }
    digitalWrite(STEPPER_EN, HIGH);
    doHome -= 4;
  }
  
}

void setup() {
  // put your setup code here, to run once:
  pinMode(SPINDLE_PWM, OUTPUT);
  pinMode(SPINDLE_DIR, OUTPUT);
  pinMode(STEPPER_EN, OUTPUT);
  digitalWrite(STEPPER_EN, HIGH);
  pinMode(X_DIR, OUTPUT);
  pinMode(X_STEP, OUTPUT);
  pinMode(Y_DIR, OUTPUT);
  pinMode(Y_STEP, OUTPUT);
  pinMode(Z_DIR, OUTPUT);
  pinMode(Z_STEP, OUTPUT);

  Serial.begin(115200);

  //doHome = 7;
  edm_active = 0;
  retract_steps = 0;
  lastRetract = 0;
}

void loop() {

  if(edm_active == 1){
    if(!digitalRead(PROBE)){
      if(digitalRead(LIMIT_Z)){
        if(retract_steps == 0){      
          for(int i = 0; i < RETRACT; i++){
            makeStepZ(0);
            delayMicroseconds(STEP_PAUSE_UP);
            retract_steps++;
          }
          //Serial.println("Retracting...");
          lastRetract = millis();
        }
        if(millis()-lastRetract > 1000){
          retract_steps = 0;
        }
      }
    }
    else{
      makeStepZ(1);
      delayMicroseconds(STEP_PAUSE_DOWN);
      retract_steps = 0;
    }
  }

  if(!digitalRead(LIMIT_Y) && edm_active == 0){
    analogWrite(SPINDLE_PWM, 40);
    delay(2500);
    digitalWrite(STEPPER_EN, LOW);
    edm_active = 1;
  }
  if(digitalRead(LIMIT_Y) && edm_active > 0){
    analogWrite(SPINDLE_PWM, 0);
    digitalWrite(STEPPER_EN, HIGH);
    delay(5000);
    edm_active = 0;
  }

  if (Serial.available() > 0) {
    int inByte = Serial.read();
    if(inByte == 'e'){
      digitalWrite(STEPPER_EN, HIGH);
      Serial.println("Steppers Disabled!");
    }
    if(inByte == 'E'){
      digitalWrite(STEPPER_EN, LOW);
      Serial.println("Steppers Enabled!");
    }
    if(inByte == 'z'){// Go up
      Serial.println("Going UP...");
      for(int i=0; i<10; i++){
        makeStepZ(0);
        delayMicroseconds(STEPPER_STEP_PAUSE);
      }      
      Serial.println("...done!");
    }
    if(inByte == 'Z'){// Go down
      Serial.println("Going Down...");
      for(int i=0; i<10; i++){
        makeStepZ(1);
        delayMicroseconds(5000);
      }      
      Serial.println("...done!");
    }
    if(inByte == 'S'){
      Serial.println("Spindle ON!");
      analogWrite(SPINDLE_PWM, 50);
      delay(1000);
      analogWrite(SPINDLE_PWM, 30);
    }
    if(inByte == 's'){
      Serial.println("Spindle OFF!");
      analogWrite(SPINDLE_PWM, 0);
    }
    if(inByte == 'A'){
      edm_active = 1;
      digitalWrite(STEPPER_EN, LOW);
      Serial.println("EDM mode enabled!");
    }
    if(inByte == 'a'){
      edm_active = 0;
      digitalWrite(STEPPER_EN, LOW);
      Serial.println("EDM mode disabled!");
    }
  }


  // homeAxes();
}
